@@ -18,6 +18,6 @@
 
 #define NL_MAJOR 2
 #define NL_MINOR 2
-#define NL_REV   7
-#define NL_DATE  "Sep 02, 2006"
-#define VERSION  "2.2.7"
+#define NL_REV   9
+#define NL_DATE  "Feb 18, 2012"
+#define VERSION  "2.2.9"
